@@ -29,6 +29,7 @@
 #define CATEGORY_MAX_LENGTH 64
 #define DEFINITION_MAX_LENGTH 1024
 #define INFORMATION_MAX_LENGTH 128
+#define READLINE_MAX_LENGTH 2048
 
 #define MAX_STRING_LENGTH 1024
 
@@ -41,9 +42,39 @@
 #define THREAD_SCAN_RUNNING_YES		2
 #define THREAD_SCAN_RUNNING_ENDED	3
 
-#define ERROR_DIALOG(msg, w) gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog_parented(_(msg), GTK_WINDOW (w))));
-#define WARNING_DIALOG(msg, w) gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog_parented(_(msg), GTK_WINDOW (w))));
+#define TEST_VFS_NO       1
+#define TEST_VFS_YES      2
+#define TEST_VFS_EXTENDED 3
 
+#define COMPARE_EXACT	1
+#define COMPARE_REGEX	2
+#define COMPARE_GLOB	3
+
+#define EVERYTHING_MATCHES 0
+#define EXACT_MATCH_CS 1
+#define EXACT_MATCH_CU 2
+#define CONTAINS_CS 3
+#define CONTAINS_CU 4
+#define GLOB_CS 5
+#define GLOB_CU 6
+#define REGULAR_EXPRESSION_CS 7
+#define REGULAR_EXPRESSION_CU 8
+#define STRING_SM_NB 9		/* Nb of strings in STRING_SM */
+
+
+#define ERROR_DIALOG(msg, w) gnome_dialog_run_and_close(GNOME_DIALOG(gnome_error_dialog_parented(msg, GTK_WINDOW (w))));
+#define WARNING_DIALOG(msg, w) gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog_parented(msg, GTK_WINDOW (w))));
+
+#define SAVE_CAT_MODE__ALL         1
+#define SAVE_CAT_MODE__ONLY_USED   2
+
+#define FDMASK_CATEGORY		0x1
+#define FDMASK_DESCRIPTION	0x2
+#define FDMASK_INFORMATION	0x4
+#define FDMASK_MIME		0x8
+
+#define CATMASK_CATEGORY	0x2
+#define CATMASK_DESCRIPTION	0x1
 
 
 /*********************** structures for the file tree ***********************/
@@ -56,6 +87,7 @@ typedef struct
   GPtrArray *categories;
 
   GtkWidget *ctree;
+  gint ctree_tree_expand_signal_id;
 
   GNode *selected_folder;
   GNode *currently_displayed_gnode;
@@ -69,15 +101,21 @@ FOLDER;
 
 typedef struct
 {
+  guint32 id;			/* index in the array containing all the FILE_DATA */
   GString *name;		/* name of the file/dir */
   guint32 taille;		/* length of the file/dir */
-  guint type;			/* type of the file/dir */
-  time_t date;			/*  time_t */
+  guint16 type;			/* type of the file/dir */
+  time_t date;			/* time_t */
+  GString *mime;		/* Mime type of the file */
   guint16 categorie;
   guint16 description;
   GString *information;
   GNode *node;			/* gnode associated with this FILE_DATA */
   GtkCTreeNode *ctree_node;	/* ctree_node associated with this FILE_DATA */
+  
+  gint a_parent_node_is_vfs;	/* This gint means that the file is inside a vfs.
+                                 * The content of this gint is the type of vfs
+				 */
 }
 FILE_DATA;
 
@@ -87,29 +125,56 @@ typedef struct
   GdkPixmap *pixmap;
   GdkBitmap **mask;
   GString *path;
+  gboolean used;	/* Warning: this is only used during the garbage collection in
+                         * setup.c. Consider this field as unset.
+			 */
 }
 PIX;
 
 typedef struct
 {
-  GString *extension;
+  GString *mime;
   GString *prog;
   GString *arg;
-  GString *viewer;
-  GString *viewerarg;
   gboolean state;
 }
-EXTENSIONS;
+MIME_EXTENSIONS;
 
 typedef struct
 {
-  GTree *icons;			/* key=extension; value=PIX */
+  GString *mime;
+  GString *prog;
+  GString *args;
+  GString *separator;
+  gint date;			/* First field is 0 */
+  GString *date_format;		/* A string like "DD-MM-YYYY" or "YY-MM-DD" for example */
+  gint size;
+  gint name;
+  gboolean state;
+}
+MIME_VFS_EXTENSIONS;
+
+typedef struct
+{
+  gchar *fullname;
+  gchar *label;
+}
+RECENT_FILE;
+
+typedef struct
+{
+  GTree *mime_icons;		/* key=extension; value=PIX */
+  GTree *pix_paths;		/* key=path; value=PIX */
   PIX *default_file_pix;
   PIX *default_dir_pix;
   PIX *default_disk_pix;
-  GTree *extensions;
+  PIX *default_unreadable_pix;
+  PIX *default_vfs_pix;
+  GTree *mime_extensions;
+  GTree *mime_vfs_extensions;
 
   gboolean getInformation;
+  gint testvfs;
 
   GtkWidget *search_dirtree;
   gboolean update_edit_window;
@@ -127,16 +192,38 @@ typedef struct
   GString *mount_point;
   GString *dummy_prog;
   gboolean use_supermount;
+  gboolean use_automount;
 
   /* Options */
   gboolean use_du_s;
   gboolean warn_mount;
   gboolean eject_disk;
+  gboolean dbclick_opens_info;
+  gboolean display_dir_size;
+  gboolean save_only_used_categories;
+  gboolean scan_mime_types;
+  gboolean use_default_information_plugin;
+  MIME_EXTENSIONS *default_information_plugin;
 
   gint compression_level;
+  
+  GList *recent_files;
+  gint nb_max_recent_files;
+  
+  gchar *date_format;
+
+  gint previous_search_mode;
+  
+  gint mainwindow_x;
+  gint mainwindow_y;
+  gint mainwindow_w;
+  gint mainwindow_h;
+  gboolean mainwindow_save_xywh;
 
 /* What is that? */
   char *fast_name;
+  char *working_path;
+
 }
 CONFIG;
 
@@ -169,7 +256,7 @@ gboolean check_supermount (CONFIG * my_config);
 void config_struct_free_1 (GList * gl);
 int yyerror (char *s);
 void yyrestart (FILE * new_file);
-CONFIG *init_config (GString * config_file, GtkWidget * w);
+CONFIG *init_config (GtkWidget * w);
 
 /* config_lex.l */
 int yylex (void);
