@@ -30,7 +30,6 @@ FILE_DATA *
 get_file_data_from_gnode (GNode * gn)
 {
   FILE_DATA *fd;
-  GNode *root_gn;
   FOLDER *racine;
 
   if (G_NODE_IS_ROOT (gn))
@@ -40,9 +39,7 @@ get_file_data_from_gnode (GNode * gn)
     }
   else
     {
-      root_gn = g_node_get_root (gn);
-      racine = root_gn->data;
-      fd = g_ptr_array_index (racine->datas, GPOINTER_TO_UINT (gn->data));
+      fd = gn->data;
     }
   return (fd);
 }
@@ -110,7 +107,8 @@ get_real_path_from_node (GNode * gn)
       tmp = tmp->parent;
     }
   fd = get_file_data_from_gnode (tmp);
-  rp = g_string_prepend (rp, fd->information->str);
+  if (fd->information)
+    rp = g_string_prepend (rp, fd->information->str);
   rp = g_string_prepend_c (rp, '/');
 
   return (rp);
@@ -133,7 +131,7 @@ get_path_from_node (GNode * gn)
 }
 
 FILE_DATA *
-folder_make_data (char *name, char *information, guint type,
+folder_make_data (char *name, char *information, guint type, gchar *mime,
 		  guint32 taille, time_t date,
 		  guint32 categorie, guint32 description)
 {
@@ -143,9 +141,11 @@ folder_make_data (char *name, char *information, guint type,
   data->name = g_string_new (name);
   data->taille = taille;
   data->type = type;
+  data->mime = g_string_new (mime);;
   data->date = date;
   data->categorie = categorie;
   data->description = description;
+  data->a_parent_node_is_vfs = type;
 
   if ((information != NULL) && (strlen (information) != 0))
     data->information = g_string_new (information);
@@ -159,26 +159,20 @@ folder_make_data (char *name, char *information, guint type,
 /* Add a file to the FOLDER structure */
 GNode *
 folder_add (FOLDER * fd, GNode * parent, char *name,
-	    char *information, guint type, guint32 taille,
+	    char *information, guint type, gchar *mime, guint32 taille,
 	    time_t date, guint32 categorie, guint32 description)
 {
 
   FILE_DATA *data;
   GNode *gn;
-/************
-printf("folder_add : name = '%s' parent = %d\n", name, GPOINTER_TO_UINT(parent->data));
-if(parent->data) {
-data = g_ptr_array_index(fd->datas, GPOINTER_TO_UINT(parent->data));
-printf(" parent name = '%s'\n\n", data->name->str);
-}
-************/
 
   data =
-    folder_make_data (name, information, type, taille, date,
+    folder_make_data (name, information, type, mime, taille, date,
 		      categorie, description);
 
   g_ptr_array_add (fd->datas, data);
-  gn = g_node_new (GUINT_TO_POINTER (fd->datas->len - 1));
+  data->id = fd->datas->len;
+  gn = g_node_new (data);
   data->node = gn;
   data->ctree_node = NULL;
   g_node_insert (parent, -1, gn);
@@ -196,9 +190,9 @@ folder_add_link (FOLDER * fd, GNode * parent,	/* parent that contains the link *
 {
   GNode *gn;
   gn =
-    folder_add (fd, parent, linkname, information, IS_LINK,
+    folder_add (fd, parent, linkname, information, IS_LINK, NULL,
 		0, date, categorie, description);
-  folder_add (fd, gn, destname, NULL, 0, 0, date, categorie, description);
+  folder_add (fd, gn, destname, NULL, 0, NULL, 0, date, categorie, description);
   return (gn);
 }
 
@@ -242,6 +236,14 @@ change_information (GNode * gn, char *information)
     fd->information = NULL;
 }
 
+void
+change_type (GNode * gn, guint type)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  fd->type = type;
+}
 
 /* traitement des infos sur la date, la taille et le type. */
 
@@ -254,6 +256,17 @@ folder_get_name (GNode * gn)
   return (fd->name->str);
 }
 
+const char *
+folder_get_mime (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->mime)
+    return (fd->mime->str);
+  return (gnome_mime_type_or_default (fd->name->str, NULL));
+}
+
 const char *
 folder_get_informations (GNode * gn)
 {
@@ -285,18 +298,29 @@ folder_get_description_from_file_data (FILE_DATA * fd)
 }
 
 char *
-get_time (time_t nb_sec)
+get_time (time_t nb_sec, char *pattern)
 {
   struct tm *time;
+  time_t nb_sec2;
   char *mydate;
-
   if (nb_sec)
     {
-      time = localtime (&nb_sec);
-      mydate = (char *) g_malloc (sizeof (char) * 25);
-      sprintf (mydate, "%.2d/%.2d/%.4d %.2d:%.2d:%.2d", time->tm_mday,
-	       time->tm_mon + 1, 1900 + time->tm_year, time->tm_sec,
-	       time->tm_min, time->tm_hour);
+      if (pattern)
+	{
+	  time = localtime (&nb_sec);
+	  mydate = (char *) g_malloc (sizeof (char) * 256);
+	  if (!strftime (mydate, 255, pattern, time))
+	    {
+	      free (mydate);
+	      memcpy(&nb_sec2, &nb_sec, sizeof(time_t));
+	      mydate = g_strdup (ctime (&nb_sec2));
+	    }
+	}
+      else
+	{
+	      memcpy(&nb_sec2, &nb_sec, sizeof(time_t));
+	  mydate = g_strdup (ctime (&nb_sec2));
+	}
     }
   else
     {
@@ -342,6 +366,23 @@ get_categorie (FOLDER * racine, guint16 i)
     }
 }
 
+gint category_id (FOLDER * racine, GString * gs)
+{
+  gint i;
+  gint id;
+  i = 0;
+  id = -1;
+  while ((i < racine->categories->len) && (id == -1))
+    {
+      if (strcmp ((((GString *)
+		    g_ptr_array_index (racine->categories, i))->str),
+		  gs->str) == 0)
+	id = i + 1;
+      i++;
+    }
+  return (id);
+}
+
 const char *
 get_description (FOLDER * racine, guint16 i)
 {
@@ -357,6 +398,16 @@ get_description (FOLDER * racine, guint16 i)
     }
 }
 
+gboolean is_unreadable (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->type == IS_UNREADABLE)
+    return (TRUE);
+  return (FALSE);
+}
+
 gboolean is_file (GNode * gn)
 {
   FILE_DATA *fd;
@@ -399,6 +450,47 @@ gboolean is_disk (GNode * gn)
   return (FALSE);
 }
 
+gboolean is_vfs (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->type == IS_VFS)
+    return (TRUE);
+  return (FALSE);
+}
+
+gboolean is_vfs_extended (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if (fd->type == IS_VFS_EXTENDED)
+    return (TRUE);
+  return (FALSE);
+}
+
+gboolean is_vfs_any (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if ((fd->type == IS_VFS) || (fd->type == IS_VFS_EXTENDED))
+    return (TRUE);
+  return (FALSE);
+}
+
+gboolean is_in_vfs_any (GNode * gn)
+{
+  FILE_DATA *fd;
+
+  fd = get_file_data_from_gnode (gn);
+  if ((fd->a_parent_node_is_vfs == IS_VFS)
+      || (fd->a_parent_node_is_vfs == IS_VFS_EXTENDED))
+    return (TRUE);
+  return (FALSE);
+}
+
 /* Find a FOLDER structure f with (f->name == name).
  * already_found sould be NULL
  * recursive is a flag to make a recursive search or only a one-level search
@@ -440,12 +532,15 @@ gboolean _gnode_clear_data (GNode * gn, gpointer data)
       fd = get_file_data_from_gnode (gn);
       if (fd->name)
 	g_string_free (fd->name, TRUE);
+      if (fd->mime)
+	g_string_free (fd->mime, TRUE);
       if (fd->information)
 	g_string_free (fd->information, TRUE);
 
-      g_free (fd);
 /* warning: the FILE_DATA entry in the GPtrArray is just set to NULL. It is not suppressed here. */
-      g_ptr_array_index (f->datas, GPOINTER_TO_UINT (gn->data)) = NULL;
+      g_ptr_array_index (f->datas, fd->id) = NULL;
+
+      g_free (fd);
     }
   return (FALSE);
 }
@@ -456,8 +551,9 @@ _gnode_clear_ctree_data (GNode * gn, gpointer data)
   FOLDER *racine = data;
   FILE_DATA *fd;
 
-  if (((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE))
-      && (G_NODE_IS_ROOT (gn) != TRUE))
+  if (
+      ((is_dir (gn) == TRUE) || (is_disk (gn) == TRUE)
+       || (is_vfs_any (gn) == TRUE)) && (G_NODE_IS_ROOT (gn) != TRUE))
     {
       fd = get_file_data_from_gnode (gn);
       if (fd->ctree_node)
@@ -475,45 +571,70 @@ suppress_dir (GNode * dir)
   FOLDER *racine;
   GPtrArray *gpa;
   FILE_DATA *fd;
-  GNode *gn;
   gint i;
 
   racine = get_catalog_from_gnode (dir);
   gpa = racine->datas;
-  fd = get_file_data_from_gnode (dir);
+
+  for (i = 0; i < gpa->len; i++)
+    {
+      fd = g_ptr_array_index (gpa, i);
+      fd->id = i;
+    }
+
+
+  fd = dir->data;
+
   if ((is_dir (dir) == TRUE) || (is_disk (dir) == TRUE)
-      || (G_NODE_IS_ROOT (dir) == TRUE))
+      || (is_vfs_any (dir) == TRUE) || (G_NODE_IS_ROOT (dir) == TRUE))
     {
       g_node_traverse (dir, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 		       _gnode_clear_ctree_data, racine);
       if (G_NODE_IS_ROOT (dir) != TRUE)
 	gtk_ctree_remove_node (GTK_CTREE (racine->ctree), fd->ctree_node);
     }
+
   g_node_traverse (dir, G_PRE_ORDER, G_TRAVERSE_ALL, -1, _gnode_clear_data,
 		   racine);
 
   if ((G_NODE_IS_ROOT (dir) == TRUE))
     {
       fd = get_file_data_from_gnode (dir);
-      if (fd->name)
-	g_string_free (fd->name, TRUE);
-      if (fd->information)
-	g_string_free (fd->information, TRUE);
+      if (fd)
+	{
+	  if (fd->name)
+	    g_string_free (fd->name, TRUE);
+	  if (fd->information)
+	    g_string_free (fd->information, TRUE);
 
-      g_free (fd);
+	  g_ptr_array_index (gpa, 0) = NULL;
+	  g_free (fd);
+	}
+      dir->data = NULL;
+    }
+  else
+    {
+      g_node_unlink (dir);
+      g_node_destroy (dir);
     }
-  g_node_unlink (dir);
-  g_node_destroy (dir);
 
   /* Garbage collector for the FOLDER data structure */
   while (g_ptr_array_remove_fast (gpa, NULL));
   for (i = 0; i < gpa->len; i++)
     {
       fd = g_ptr_array_index (gpa, i);
-      gn = fd->node;
-      gn->data = GUINT_TO_POINTER (i);
+      fd->id = i;
+    }
+
+  if ((G_NODE_IS_ROOT (dir) == TRUE))
+    {
+      g_node_destroy (dir);
+
+    }
+  else
+    {
+      racine->tree->data = racine;
     }
-  racine->tree->data = racine;
   racine->selected_folder = NULL;
   racine->is_modified = TRUE;
 }
@@ -532,7 +653,7 @@ _make_disk_list (GNode * gn, gpointer data)
 
   FILE_DATA *fd;
 
-  fd = g_ptr_array_index (disk_list->f->datas, GPOINTER_TO_UINT (gn->data));
+  fd = gn->data;
 
   disk_list->list = g_list_append (disk_list->list, fd->name->str);
   return;
@@ -574,7 +695,6 @@ gboolean _test_description (GNode * gn, gpointer data)
     {
       if (g_tree_lookup ((GTree *) data, GUINT_TO_POINTER (u)) == NULL)
 	{
-	  printf ("test %d\n", u);
 	  g_ptr_array_add (new_descriptions,
 			   (gp =
 			    g_ptr_array_index (my_config->
@@ -632,3 +752,161 @@ description_garbage_collection ()
   g_ptr_array_free (racine->descriptions, FALSE);
   racine->descriptions = new_descriptions;
 }
+
+FOLDER *
+init_folder ()
+{
+  FILE_DATA *fd;
+  FOLDER *racine;
+
+  racine = (FOLDER *) g_malloc (sizeof (FOLDER));
+  racine->datas = g_ptr_array_new ();
+  fd = (FILE_DATA *) g_malloc (sizeof (FILE_DATA));
+  g_ptr_array_add (racine->datas, fd);
+  racine->tree = g_node_new (racine);
+  fd->name = NULL;
+  fd->taille = 0;
+  fd->type = 0;
+  fd->mime = NULL;
+  fd->date = 0;
+  fd->categorie = 0;
+  fd->description = 0;
+  fd->information = NULL;
+  fd->ctree_node = NULL;
+  fd->a_parent_node_is_vfs = IS_DISK;
+
+
+  racine->categories = g_ptr_array_new ();
+
+  racine->descriptions = g_ptr_array_new ();
+  racine->catalog_filename = g_string_new (_("No_name"));
+  racine->catalog_filename_is_valid = FALSE;
+  racine->selected_folder = NULL;
+  racine->currently_displayed_gnode = NULL;
+  racine->is_modified = FALSE;
+
+  racine->ctree = NULL;
+
+  fd->node = racine->tree;
+
+  return (racine);
+}
+
+void
+clear_folder (FOLDER * racine)
+{
+  gint i;
+
+  if (racine->tree)
+    {
+      suppress_dir (racine->tree);
+    }
+
+  if (racine->descriptions)
+    {
+      for (i = 0; i < racine->descriptions->len; i++)
+	{
+	  g_string_free (g_ptr_array_index (racine->descriptions, i), TRUE);
+	}
+      g_ptr_array_free (racine->descriptions, FALSE);
+    }
+
+  if (racine->categories)
+    {
+      for (i = 0; i < racine->categories->len; i++)
+	{
+	  g_string_free (g_ptr_array_index (racine->categories, i), TRUE);
+	}
+      g_ptr_array_free (racine->categories, FALSE);
+    }
+
+  if (racine->catalog_filename)
+    g_string_free (racine->catalog_filename, TRUE);
+
+  if (racine->ctree)
+    gtk_widget_destroy (GTK_WIDGET (racine->ctree));
+
+  g_free (racine);
+
+}
+
+gboolean
+_scan_folders_for_stats (GNode * gn, gpointer data)
+{
+  guint32 *stats = data;
+  FILE_DATA *fd;
+
+  if (is_dir (gn))
+    {
+      stats[2]++;
+      if (is_in_vfs_any (gn))
+	stats[0]++;
+    }
+  else if (!is_disk (gn))
+    {
+      stats[3]++;
+      if (is_in_vfs_any (gn))
+	stats[1]++;
+      else if (is_file (gn))
+	{
+	  fd = get_file_data_from_gnode (gn);
+	  stats[5] += (fd->taille % 1048576);	/* 1024^2 = 1048576 */
+	  stats[4] += (fd->taille / 1048576) + (stats[5] / 1048576);
+	  stats[5] = (stats[5] % 1048576);
+
+	}
+      if (is_vfs_any (gn))
+	stats[6]++;
+    }
+  return (FALSE);
+}
+
+void
+folder_statistics (FOLDER * racine, GNode *gn, guint32 * nb_desc, guint32 * nb_cat,
+		   guint32 * nb_disks, guint32 * nb_dirs,
+		   guint32 * nb_dirs_with_vfs, guint32 * nb_files,
+		   guint32 * nb_files_with_vfs, guint32 * total_size,
+		   guint32 * total_size_2, guint32 * nb_vfs)
+{
+  guint32 stats[7] = { 0, 0, 0, 0, 0, 0, 0 };
+  /* stats[0] = nb of dirs only if inside vfs
+   * stats[1] = nb of files only if inside vfs
+   * stats[2] = nb of dirs including those inside vfs
+   * stats[3] = nb of files including those inside vfs
+   * stats[4] = Total size (Mo)
+   * stats[5] = Total size(Mo) - Total size
+   * stats[6] = Nb of VFS
+   */
+  if (nb_desc)
+    {
+      *nb_desc = racine->descriptions->len;
+    }
+  if (nb_cat)
+    {
+      *nb_desc = racine->categories->len;
+    }
+  if (nb_disks)
+    {
+      *nb_disks = g_node_n_children (racine->tree);
+    }
+  if (nb_dirs || nb_dirs_with_vfs || nb_files || nb_files_with_vfs
+      || total_size)
+    {
+      g_node_traverse (gn, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+		       _scan_folders_for_stats, stats);
+      if (nb_dirs)
+	*nb_dirs = stats[2] - stats[0];
+      if (nb_dirs_with_vfs)
+	*nb_dirs_with_vfs = stats[2];
+      if (nb_files)
+	*nb_files = stats[3] - stats[1] - 1;
+      if (nb_files_with_vfs)
+	*nb_files_with_vfs = stats[3] - 1;
+      if (total_size)
+	*total_size = stats[4];
+      if (total_size_2)
+	*total_size_2 = stats[5];
+      if (nb_vfs)
+	*nb_vfs = stats[6];
+    }
+}
