@@ -24,11 +24,14 @@
 #include "config_common.h"
 
 
-#define IS_DISK        1
-#define IS_DIR         2
-#define IS_FILE        3
-#define IS_LINK        4
-#define IS_ENDOFDIR    12
+#define IS_DISK          1
+#define IS_DIR           2
+#define IS_FILE          3
+#define IS_LINK          4
+#define IS_VFS           5
+#define IS_VFS_EXTENDED  6
+#define IS_UNREADABLE    7
+#define IS_ENDOFDIR      12
 
 FILE_DATA *get_file_data_from_gnode (GNode * gn);
 FOLDER *get_catalog_from_file_data (FILE_DATA * fd);
@@ -38,11 +41,11 @@ void catalog_set_currently_displayed_gnode (GNode * gn);
 const gchar *get_disk_from_node (GNode * gn);
 GString *get_real_path_from_node (GNode * gn);
 GString *get_path_from_node (GNode * gn);
-FILE_DATA *folder_make_data (char *name, char *information, guint type,
+FILE_DATA *folder_make_data (char *name, char *information, guint type, gchar *mime,
 			     guint32 taille, time_t date, guint32 categorie,
 			     guint32 description);
 GNode *folder_add (FOLDER * fd, GNode * parent, char *name, char *information,
-		   guint type, guint32 taille, time_t date, guint32 categorie,
+		   guint type, gchar *mime, guint32 taille, time_t date, guint32 categorie,
 		   guint32 description);
 GNode *folder_add_link (FOLDER * fd, GNode * parent, char *linkname,
 			char *destname, char *information, guint32 taille,
@@ -51,21 +54,37 @@ void change_name (GNode * gn, char *name);
 void change_description (GNode * gn, guint i);
 void change_category (GNode * gn, guint i);
 void change_information (GNode * gn, char *information);
+void change_type (GNode * gn, guint type);
 const char *folder_get_name (GNode * gn);
+const char *folder_get_mime (GNode * gn);
 const char *folder_get_informations (GNode * gn);
 const char *folder_get_category_from_file_data (FILE_DATA * fd);
 const char *folder_get_description_from_file_data (FILE_DATA * fd);
-char *get_time (time_t nb_sec);
+char *get_time (time_t nb_sec, char *pattern);
 char *get_size (guint32 nb_octs);
 int get_type (guint type_info);
 const char *get_categorie (FOLDER * racine, guint16 i);
+gint category_id (FOLDER * racine, GString * gs);
 const char *get_description (FOLDER * racine, guint16 i);
+gboolean is_unreadable (GNode * gn);
 gboolean is_file (GNode * gn);
 gboolean is_dir (GNode * gn);
 gboolean is_link (GNode * gn);
 gboolean is_disk (GNode * gn);
+gboolean is_vfs (GNode * gn);
+gboolean is_vfs_extended (GNode * gn);
+gboolean is_vfs_any (GNode * gn);
+gboolean is_in_vfs_any (GNode * gn);
 GArray *folder_search_name_in_array (GNode * gn, gchar * name);
 void suppress_dir (GNode * dir);
 GList *make_disk_list (FOLDER * racine);
 void description_garbage_collection ();
+gint _cmp_intptrs (gconstpointer a, gconstpointer b);
+FOLDER *init_folder(void);
+void clear_folder(FOLDER *racine);
+void folder_statistics (FOLDER * racine, GNode *gn, guint32 * nb_desc, guint32 * nb_cat,
+			guint32 * nb_disks, guint32 * nb_dirs,
+			guint32 * nb_dirs_with_vfs, guint32 * nb_files,
+			guint32 * nb_files_with_vfs, guint32 * total_size,
+			guint32 * total_size_2, guint32 * nb_vfs);
 #endif
