@@ -24,6 +24,53 @@
 #include "folder.h"
 
 static GtkWidget *fs;
+static GtkWidget *toggle_save_cat_mode;
+
+
+PIX *
+get_pix_from_gnode (GNode * gn)
+{
+  gchar *mime = NULL;
+  FILE_DATA *fd;
+  PIX *pix;
+
+  if (is_unreadable (gn) == TRUE)
+    {
+      pix = my_config->default_unreadable_pix;
+    }
+  else
+    {
+      fd = get_file_data_from_gnode (gn);
+
+
+      if (fd->mime)
+	mime = g_strdup (fd->mime->str);
+      if (!mime)
+	mime = g_strdup (gnome_mime_type_or_default (fd->name->str, NULL));
+      if (mime)
+	{
+	  pix = ((PIX *) g_tree_lookup (my_config->mime_icons, mime));
+	  if (!pix)
+	    {
+	      if (is_dir (gn) == TRUE)
+		pix = my_config->default_dir_pix;
+	      else
+		pix = my_config->default_file_pix;
+
+	    }
+	  g_free (mime);
+	}
+      else
+	{
+	  if (is_dir (gn) == TRUE)
+	    pix = my_config->default_dir_pix;
+	  else
+	    pix = my_config->default_file_pix;
+
+	}
+    }
+  return (pix);
+}
 
 /* This traversefunc creates a gtkctree_node with the data given with gn.
  * This gtkctree_node is inserted to data->ctree where data is FOLDER*.
@@ -35,7 +82,7 @@ gboolean add_gnode_to_ctree (GNode * gn, gpointer data)
   FOLDER *racine = data;
   FILE_DATA *fd, *parent_fd, *child_fd;
   GtkCTreeNode *gtkparentnode;
-  gboolean is_leaf, is_expanded;
+  gboolean is_leaf;
   GNode *gn_child;
   PIX *pix;
 
@@ -47,28 +94,29 @@ gboolean add_gnode_to_ctree (GNode * gn, gpointer data)
       while ((gn_child) && (is_leaf == TRUE))
 	{
 	  child_fd = get_file_data_from_gnode (gn_child);
-	  if (child_fd->type == IS_DIR)
+	  if ((child_fd->type == IS_DIR) || (child_fd->type == IS_VFS)
+	      || (child_fd->type == IS_VFS_EXTENDED))
 	    is_leaf = FALSE;
 	  gn_child = g_node_next_sibling (gn_child);
 	}
-      if ((racine->tree == gn))
-	is_expanded = TRUE;
-      else
-	is_expanded = FALSE;
 
-      if ((is_disk (gn) == TRUE) || (is_dir (gn) == TRUE))
+      if ((is_disk (gn) == TRUE) || (is_dir (gn) == TRUE)
+	  || (is_vfs_any (gn) == TRUE))
 	{
 	  parent_fd = get_file_data_from_gnode (gn->parent);
 	  gtkparentnode = parent_fd->ctree_node;
 	  if (g_node_depth (gn) == 2)
 	    pix = my_config->default_disk_pix;
+	  else if (is_vfs_any (gn) == TRUE)
+	    pix = my_config->default_vfs_pix;
 	  else
 	    pix = my_config->default_dir_pix;
 	  fd->ctree_node =
 	    gtk_ctree_insert_node (GTK_CTREE (racine->ctree),
 				   gtkparentnode, NULL, &(fd->name->str), 4,
 				   pix->pixmap, *(pix->mask), pix->pixmap,
-				   *(pix->mask), is_leaf, is_expanded);
+				   *(pix->mask), is_leaf, FALSE);
+
 	  gtk_ctree_node_set_row_data (GTK_CTREE (racine->ctree),
 				       fd->ctree_node, gn);
 
@@ -120,6 +168,102 @@ expand_cb (GtkWidget * widget, GtkCTreeNode * parent_node, FOLDER * racine)
     }
 }
 
+static gint
+_clist_g_strnatcmp (GtkCList * clist, gconstpointer ptr1, gconstpointer ptr2)
+{
+  char *text1 = NULL;
+  char *text2 = NULL;
+  char *p1, *p2;
+  gint n1, n2;
+  gint result;
+
+  GtkCListRow *row1 = (GtkCListRow *) ptr1;
+  GtkCListRow *row2 = (GtkCListRow *) ptr2;
+
+  switch (row1->cell[clist->sort_column].type)
+    {
+    case GTK_CELL_TEXT:
+      text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
+      break;
+    case GTK_CELL_PIXTEXT:
+      text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
+      break;
+    default:
+      break;
+    }
+
+  switch (row2->cell[clist->sort_column].type)
+    {
+    case GTK_CELL_TEXT:
+      text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
+      break;
+    case GTK_CELL_PIXTEXT:
+      text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
+      break;
+    default:
+      break;
+    }
+
+  if (!text2)
+    return (text1 != NULL);
+
+  if (!text1)
+    return -1;
+
+  result = 0;
+  p1 = text1;
+  p2 = text2;
+  while ((p1[0] != 0) && (p1[0] != 0) && (result == 0))
+    {
+      if ((p1[0] >= '0') && (p1[0] <= '9') && (p2[0] >= '0')
+	  && (p2[0] <= '9'))
+	{
+	  n1 = strtol (p1, NULL, 10);
+	  n2 = strtol (p2, NULL, 10);
+	  if (n1 > n2)
+	    result = 1;
+	  if (n1 < n2)
+	    result = -1;
+	  while ((p1[0] >= '0') && (p1[0] <= '9'))
+	    p1++;
+	  while ((p2[0] >= '0') && (p2[0] <= '9'))
+	    p2++;
+	}
+      else
+	{
+	  result = strncmp (p1, p2, 1);
+	  p1++;
+	  p2++;
+	}
+    }
+  if (p1[0] && (!p2[0]))
+    result = -1;
+  if (p2[0] && (!p1[0]))
+    result = 1;
+
+  return (result);
+}
+
+void
+expand_folder (GNode * gn)
+{
+  FOLDER *racine;
+  FILE_DATA *fd;
+  racine = get_catalog_from_gnode (gn);
+
+  gtk_signal_disconnect (GTK_OBJECT (racine->ctree),
+			 racine->ctree_tree_expand_signal_id);
+  while (g_node_depth (gn) >= 2)
+    {
+      fd = get_file_data_from_gnode (gn);
+      gtk_ctree_expand (GTK_CTREE (racine->ctree), fd->ctree_node);
+      gn = gn->parent;
+    }
+  racine->ctree_tree_expand_signal_id =
+    gtk_signal_connect (GTK_OBJECT (racine->ctree), "tree_expand",
+			GTK_SIGNAL_FUNC (expand_cb), racine);
+}
+
 
 /* Create the folder tree */
 void
@@ -132,8 +276,9 @@ create_tree (FOLDER * racine)
 				GTK_SELECTION_SINGLE);
   gtk_ctree_set_line_style (GTK_CTREE (racine->ctree),
 			    GTK_CTREE_LINES_DOTTED);
-  gtk_signal_connect (GTK_OBJECT (racine->ctree), "tree_expand",
-		      GTK_SIGNAL_FUNC (expand_cb), racine);
+  racine->ctree_tree_expand_signal_id =
+    gtk_signal_connect (GTK_OBJECT (racine->ctree), "tree_expand",
+			GTK_SIGNAL_FUNC (expand_cb), racine);
   gtk_signal_connect (GTK_OBJECT (racine->ctree), "tree_select_row",
 		      GTK_SIGNAL_FUNC (cb_itemsignal), racine);
   gtk_signal_connect (GTK_OBJECT (racine->ctree), "tree_unselect_row",
@@ -143,45 +288,14 @@ create_tree (FOLDER * racine)
 		      GTK_SIGNAL_FUNC (tree_pressed), racine);
 
   gtk_object_set_user_data (GTK_OBJECT (racine->ctree), racine->tree);
+
+  gtk_clist_set_compare_func (GTK_CLIST (racine->ctree), _clist_g_strnatcmp);
+  gtk_clist_set_sort_column (GTK_CLIST (racine->ctree), 0);
+  gtk_clist_set_sort_type (GTK_CLIST (racine->ctree), GTK_SORT_ASCENDING);
 }
 
 
 
-FOLDER *
-init_folder ()
-{
-  FILE_DATA *fd;
-  FOLDER *racine;
-
-  racine = (FOLDER *) g_malloc (sizeof (FOLDER));
-  racine->datas = g_ptr_array_new ();
-  fd = (FILE_DATA *) g_malloc (sizeof (FILE_DATA));
-  g_ptr_array_add (racine->datas, fd);
-  racine->tree = g_node_new (racine);
-  fd->name = NULL;
-  fd->taille = 0;
-  fd->type = 0;
-  fd->date = 0;
-  fd->categorie = 0;
-  fd->description = 0;
-  fd->information = NULL;
-  fd->ctree_node = NULL;
-
-  racine->categories = g_ptr_array_new ();
-
-  racine->descriptions = g_ptr_array_new ();
-  racine->catalog_filename = g_string_new (_("No_name"));
-  racine->catalog_filename_is_valid = FALSE;
-  racine->selected_folder = NULL;
-  racine->currently_displayed_gnode = NULL;
-  racine->is_modified = FALSE;
-
-  create_tree (racine);
-  fd->node = racine->tree;
-
-  return (racine);
-}
-
 void
 new_file ()
 {
@@ -209,39 +323,15 @@ new_file ()
 	}
       gtk_entry_set_text (GTK_ENTRY (path_entry), "");
       my_config->racine = init_folder ();
-      add_gnode_to_ctree (my_config->racine->tree, my_config->racine);
+      create_tree (my_config->racine);
+
       ctree_window_add (my_config->racine->ctree);
       update_tree (my_config->racine);
+
     }
-  /*
-   * gtk_statusbar_pop ((GtkStatusbar *) status_bar1, status_context1);
-   * gtk_statusbar_push ((GtkStatusbar *) status_bar1, status_context1, ""); 
-   */
 
 }
 
-void
-clear_folder (FOLDER * racine)
-{
-  gint i;
-  suppress_dir (racine->tree);
-
-  for (i = 0; i < racine->descriptions->len; i++)
-    {
-      g_string_free (g_ptr_array_index (racine->descriptions, i), TRUE);
-    }
-  g_ptr_array_free (racine->descriptions, FALSE);
-
-  for (i = 0; i < racine->categories->len; i++)
-    {
-      g_string_free (g_ptr_array_index (racine->categories, i), TRUE);
-    }
-  g_ptr_array_free (racine->categories, FALSE);
-  g_string_free (racine->catalog_filename, TRUE);
-  gtk_widget_destroy (GTK_WIDGET (racine->ctree));
-  g_free (racine);
-}
-
 /*
  * ouverture d'un catalogue 
  */
@@ -249,26 +339,36 @@ void
 open_cat_ok_sel (GtkWidget * w, gpointer data)
 {
   FOLDER *racine = data;
+  gchar *s, *p;
 
   gtk_clist_clear ((GtkCList *) clist);
+
   if (racine != NULL)
     {
       clear_folder (racine);
     }
+
   gtk_entry_set_text (GTK_ENTRY (path_entry), "");
+
   racine = init_folder ();
+  create_tree (racine);
+
   my_config->racine = racine;
 
-  /*
-   * gtk_statusbar_pop ((GtkStatusbar *) status_bar1, status_context1);
-   * gtk_statusbar_push ((GtkStatusbar *) status_bar1, status_context1, ""); 
-   */
-  if (load_cat_from_file
-      (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)),
-       racine) == -1)
+  s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
+  if (load_cat_from_file (s, racine) == -1)
     {
       return;
     }
+  g_free (my_config->working_path);
+  my_config->working_path = g_strdup (s);
+  p = my_config->working_path;
+  while (p[0])
+    p++;
+  while (p[0] != '/')
+    p--;
+  p[1] = 0;
+
   gtk_widget_destroy (GTK_WIDGET (fs));
   g_node_traverse (racine->tree,
 		   G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
@@ -299,16 +399,15 @@ open_cat (FOLDER * racine)
 	}
     }
   fs = gtk_file_selection_new ("File selection");
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+				   my_config->working_path);
   gtk_signal_connect (GTK_OBJECT (fs), "destroy",
 		      (GtkSignalFunc) gtk_widget_destroy, &(fs));
-  gtk_signal_connect (GTK_OBJECT
-		      (GTK_FILE_SELECTION
-		       (fs)->ok_button), "clicked",
-		      (GtkSignalFunc) open_cat_ok_sel, racine);
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
+		      "clicked", (GtkSignalFunc) open_cat_ok_sel, racine);
   gtk_signal_connect_object (GTK_OBJECT
-			     (GTK_FILE_SELECTION
-			      (fs)->cancel_button), "clicked",
-			     (GtkSignalFunc) gtk_widget_destroy,
+			     (GTK_FILE_SELECTION (fs)->cancel_button),
+			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
 			     GTK_OBJECT (fs));
   gtk_widget_show (fs);
 
@@ -320,30 +419,51 @@ open_cat (FOLDER * racine)
 void
 save_cat_ok_sel (GtkWidget * w, FOLDER * racine)
 {
-  if (save_cat_to_file
-      (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)),
-       racine) == 0)
+  gchar *s, *p;
+  s = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
+  my_config->save_only_used_categories =
+    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle_save_cat_mode));
+  racine->catalog_filename_is_valid = FALSE;
+  if (save_cat_to_file (s, racine) == 0)
     {
       gtk_window_set_title (GTK_WINDOW (main_window),
 			    racine->catalog_filename->str);
-      /*
-       * gtk_statusbar_push ((GtkStatusbar *) status_bar1, status_context1, racine->catalog_filename->str); 
-       */
+      g_free (my_config->working_path);
+      my_config->working_path = g_strdup (s);
+      p = my_config->working_path;
+      while (p[0])
+	p++;
+      while (p[0] != '/')
+	p--;
+      p[1] = 0;
     }
   gtk_widget_destroy (GTK_WIDGET (fs));
 }
 
 
 
-
 void
 save_cat (FOLDER * racine)
 {
+  GtkWidget *frame;
   fs = gtk_file_selection_new ("Save catalog to file");
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
+				   my_config->working_path);
+  frame = gtk_frame_new (_("Options"));
+  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), frame,
+		      FALSE, TRUE, 0);
+  gtk_widget_show (frame);
+
+  toggle_save_cat_mode =
+    gtk_check_button_new_with_label (_("Save only used categories"));
+  gtk_container_add (GTK_CONTAINER (frame), toggle_save_cat_mode);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle_save_cat_mode),
+				my_config->save_only_used_categories);
+  gtk_widget_show (toggle_save_cat_mode);
+
   gtk_signal_connect (GTK_OBJECT (fs), "destroy",
 		      (GtkSignalFunc) gtk_widget_destroy, &(fs));
-  gtk_signal_connect (GTK_OBJECT
-		      (GTK_FILE_SELECTION (fs)->ok_button),
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button),
 		      "clicked", (GtkSignalFunc) save_cat_ok_sel, racine);
   gtk_signal_connect_object (GTK_OBJECT
 			     (GTK_FILE_SELECTION (fs)->cancel_button),
@@ -367,7 +487,7 @@ fast_save (FOLDER * racine)
 	  /*
 	   * canceled = 1; 
 	   */
-	  ERROR_DIALOG ("Can't write file", main_window);
+	  ERROR_DIALOG (_("Can't write file"), main_window);
 	}
     }
 }
@@ -397,3 +517,31 @@ string_to_index (GPtrArray * a, gchar * s)
     }
   return (result);
 }
+
+gboolean test_plugin_existence (GString * filename)
+{
+  gchar *path;
+  gchar **paths, **p1;
+  gchar *fullname = NULL;
+  if (filename->str[0] == '/')
+    return (g_file_test (filename->str, G_FILE_TEST_ISFILE));
+  path = getenv ("PATH");
+  paths = g_strsplit (path, ":", 0);
+  p1 = paths;
+  while (*p1)
+    {
+      fullname = g_strconcat (*p1, "/", filename->str, NULL);
+      if (g_file_test (fullname, G_FILE_TEST_ISFILE))
+	break;
+      g_free (fullname);
+      fullname = NULL;
+      p1++;
+    }
+  g_strfreev (paths);
+  if (fullname)
+    {
+      g_free (fullname);
+      return (TRUE);
+    }
+  return (FALSE);
+}
