@@ -18,15 +18,19 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <string.h>
 
 #define __DEF_CONFIG__
 #include "addisk.h"
+#include "buttonbar.h"
 #include "config_common.h"
 #include "exit.h"
+#include "folder.h"
 #include "interface.h"
 #include "io.h"
 #include "loadcat.h"
-#include "buttonbar.h"
+#include "setup.h"
+#include "thread_utils.h"
 
 #define P_FILE		100
 #define P_GEOMETRY	101
@@ -54,18 +58,55 @@ struct poptOption command_line[] = {
 
 GtkWidget *main_window;
 
+static gint
+save_session (GnomeClient * client, gint phase, GnomeSaveStyle save_style,
+	      gint is_shutdown, GnomeInteractStyle interact_style,
+	      gint is_fast, gpointer client_data)
+{
+  gchar **argv;
+  guint argc;
+
+  /* allocate 0-filled, so it will be NULL-terminated */
+  argv = g_malloc0 (sizeof (gchar *) * 4);
+  argc = 1;
+
+  argv[0] = client_data;
+
+  if (my_config->racine)
+    {
+      if (my_config->racine->catalog_filename_is_valid)
+	{
+	  argv[1] = "-c";
+	  argv[2] = g_strdup (my_config->racine->catalog_filename->str);
+	  argc = 3;
+	}
+    }
+  gnome_client_set_clone_command (client, argc, argv);
+  gnome_client_set_restart_command (client, argc, argv);
+
+  return TRUE;
+}
+
+static void
+session_die (GnomeClient * client, gpointer client_data)
+{
+  quit_prog (my_config->racine);
+}
+
+
 int
 main (int argc, char *argv[])
 {
-  GString *conf_file;
   poptContext optCon;
   GString *_title = NULL;
   GString *_filename = NULL;
   GString *_geometry = NULL;
   gint wx, wy, ww, wh;
   int c;
+  GnomeClient *client;
+  gchar *cwd, *fullname_prog;
 
-
+  cwd = g_get_current_dir ();
   bindtextdomain (PACKAGE, GNOMELOCALEDIR);
   textdomain (PACKAGE);
   gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv, command_line, 0,
@@ -74,7 +115,7 @@ main (int argc, char *argv[])
 
   /* Processing of arguments */
 
-  while ((c = poptGetNextOpt (optCon)) >= 0)
+  while ((c = poptGetNextOpt (optCon)) != -1)
     {
       switch (c)
 	{
@@ -107,6 +148,7 @@ main (int argc, char *argv[])
 
   /*initialisation des differentes variables */
 
+  client = gnome_master_client ();
   main_window = gnome_app_new (PACKAGE, "gtktalog");
 
   if (_title)
@@ -118,6 +160,15 @@ main (int argc, char *argv[])
     {
       gtk_window_set_title (GTK_WINDOW (main_window), "GTKtalog V " VERSION);
     }
+  gtk_widget_realize (main_window);
+
+  my_config = init_config (GTK_WIDGET (main_window));
+  my_config->update_edit_window = FALSE;
+  my_config->calendar_widgets = NULL;
+  my_config->fast_name = NULL;
+  my_config->racine = NULL;
+  my_config->working_path = g_strconcat (getenv ("HOME"), "/", NULL);
+
   if (_geometry)
     {
       if (gnome_parse_geometry (_geometry->str, &wx, &wy, &ww, &wh) != TRUE)
@@ -127,14 +178,15 @@ main (int argc, char *argv[])
 	  ww = 660;
 	  wh = 350;
 	}
+	printf("%d %d %d %d\n", wx, wy, ww, wh);
       g_string_free (_geometry, TRUE);
     }
   else
     {
-      wx = 0;
-      wy = 0;
-      ww = 660;
-      wh = 350;
+      wx = my_config->mainwindow_x;
+      wy = my_config->mainwindow_y;
+      ww = my_config->mainwindow_w;
+      wh = my_config->mainwindow_h;
     }
   if (wx < 0)
     wx = 0;
@@ -144,36 +196,26 @@ main (int argc, char *argv[])
     ww = 660;
   if (wh < 0)
     wh = 350;
-  gtk_widget_set_uposition (GTK_WIDGET (main_window), wx, wy);
-  gtk_widget_set_usize (GTK_WIDGET (main_window), ww, wh);
 
-  gtk_widget_realize (main_window);
-
-  conf_file = g_string_new (g_get_home_dir ());
-  conf_file =
-    g_string_append (conf_file, USER_CONFIG_DIR "/" USER_CONFIG_FILE);
-  if (!g_file_test (conf_file->str, G_FILE_TEST_ISFILE))
-    {
-      g_string_free (conf_file, TRUE);
-      conf_file = g_string_new (PREFIX "/share/gtktalog/gtktalog.cf");
-      if (!g_file_test (conf_file->str, G_FILE_TEST_ISFILE))
-	{
-	  ERROR_DIALOG ("Could not find config file", main_window);
-	  exit (-1);
-	}
-    }
-
-  my_config = init_config (conf_file, GTK_WIDGET (main_window));
-  my_config->racine = NULL;
-  my_config->update_edit_window = FALSE;
-  my_config->calendar_widgets = NULL;
-  my_config->fast_name = NULL;
-  my_config->racine = NULL;
   reinit_thread_status ();
 
   build_gui ();
 
+  fullname_prog = argv[0];
+  if (!g_path_is_absolute (fullname_prog))
+    {
+      fullname_prog = g_concat_dir_and_file (cwd, fullname_prog);
+    }
+  g_free (cwd);
+
+  gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
+		      GTK_SIGNAL_FUNC (save_session), fullname_prog);
+  gtk_signal_connect (GTK_OBJECT (client), "die",
+		      GTK_SIGNAL_FUNC (session_die), NULL);
+
   gtk_widget_show (main_window);
+  if ((my_config->mainwindow_save_xywh) || (_geometry))
+    gdk_window_move_resize (main_window->window, wx, wy, ww, wh);
 
   my_config->current_pid = getpid ();
 
@@ -181,6 +223,7 @@ main (int argc, char *argv[])
     {
       gtk_entry_set_text (GTK_ENTRY (path_entry), "");
       my_config->racine = init_folder ();
+      create_tree (my_config->racine);
       load_cat_from_file (_filename->str, my_config->racine);
       g_node_traverse (my_config->racine->tree,
 		       G_PRE_ORDER, G_TRAVERSE_ALL, -1, add_gnode_to_ctree,
