@@ -20,8 +20,14 @@
 #define __PROGRESS_H__
 
 #include <config.h>
+#ifdef OS_LINUX
+#  include <sys/vfs.h>		
+#endif
+#ifdef OS_FREEBSD
+#    include <sys/param.h>
+#    include <sys/mount.h>
+#endif
 #include <gnome.h>
-#include <sys/vfs.h>		// For statfs
 
 
 /* Return the size of the filesystem
